/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file emutypes.h
 *
 * @brief Defines the data types used by the SMR Emulator.
 *
 * @author Anand Suresh <anandsuresh@cmu.edu>
 */

#ifndef __EMUTYPES_H_
#define __EMUTYPES_H_

#include <stdint.h>



/* -- Type Definitions -- */

/**
 * @brief Defines a data type for passing band numbers. All band numbers are
 * 32-bit values.
 */
typedef int32_t band_t;


/**
 * @brief Defines a data type for passing Relative Block Addresses. All RBAs
 * are 32-bit values.
 */
typedef int32_t rba_t;


/**
 * @brief Defines the types of inquiry operations that can be used with
 * inquiry().
 *
 * @param INQ_BLKSIZE Returns the size of a Block.
 * @param INQ_BNDSIZE Returns the size of a Band.
 * @param INQ_BNDCOUNT Returns the no. of a Bands on the EDI.
 */
typedef enum {
    INQ_BLKSIZE,
    INQ_BNDSIZE,
    INQ_BNDCOUNT
} inq_t;


/**
 * @brief Defines the Service Actions that can be used with managebands().
 *
 * @param SVC_GETPTR Retrieves the Write Pointer for a band
 * @param SVC_SETPTR Sets the Write Pointer for a band
 * @param SVC_GETMULTIPLE Returns the Write Pointer for multiple bands
 * @param SVC_RESETALL Resets the Write Pointers for all bands to 0
 * @param SVC_GETBANDCONFIG Gets the Band Configuration for a band
 * @param SVC_SETBANDCONFIG Sets the Band Configuration for a band
 */
typedef enum {
    SVC_GETPTR = 0x01,
    SVC_SETPTR = 0x02,
    SVC_GETMULTIPLE = 0x05,
    SVC_RESETALL = 0x06,
    SVC_GETBANDCONFIG = 0x15,
    SVC_SETBANDCONFIG = 0x16
} svc_act_t;



#endif /* __EMUTYPES_H_ */
