/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file ediversion.h
 *
 * @brief Defines macros, data types and function prototypes pertaining to
 * emulator versioning.
 *
 * @author Anand Suresh <anandsuresh@cmu.edu>
 */

#ifndef __EMUVERSION_H_
#define __EMUVERSION_H_

#include <stdarg.h>

#include "emuerror.h"
#include "emutypes.h"
#include "emudiskimg.h"



/* -- Macro Definitions -- */

/**
 * @brief The highest EDI Format version number implemented
 */
#define MAX_EDI_VERSION	0



/* -- Type Definitions -- */
typedef struct {
	int (*create)(int, va_list);
	int (*dump)(int);
    int (*open)(int);
    int (*close)(int);
    int (*modesense)(int, inq_t, void*);
    int (*managebands)(int, svc_act_t, band_t*, rba_t*, int);
#ifdef _EDI_ON_RAW
    int (*read)(int, int, band_t, rba_t, char*, rba_t);
    int (*write)(int, int, band_t, rba_t, char*, rba_t);
#else
    int (*read)(int, band_t, rba_t, char*, rba_t);
    int (*write)(int, band_t, rba_t, char*, rba_t);
#endif
}edi_vops_t;



/* -- Global Variables -- */
edi_vops_t* edi_versions[MAX_EDI_VERSION + 1];



/* -- Function Prototypes -- */
int ediv_init();
int ediv_is_valid(uint32_t);



#endif /* __EMUVERSION_H_ */
