/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file band_log.h
 *
 * @brief Defines function prototypes pertaining to Bands Logs
 *
 * @author Jainam Shah <jashah@andrew.cmu.edu>
 */

#ifndef __BAND_LOG_H_
#define __BAND_LOG_H_
#ifdef __cplusplus
 extern "C" {
#endif
#include "emutypes.h"

/*
 * Log entry structure to be appended in store log for each band when a file is 
 * placed on a band.
 *
 */

struct store_entry {
    int offset;
    int size;
    int inode_num;
    long int age;
};


/* 
 * Log entry structure to be appended in deleted log for each file deleted 
 * from a band.
 *
 */

struct delete_entry {
    int offset;
    int size;
    int entrynumber;
};



/* ========================= */
/* -- Function Prototypes -- */
/* ========================= */

int init_storelog(int band_num);
int init_deletelog(int band_num);
void clear_storelog(int band_num);
void clear_deletelog(int band_num);
int append_storelog(band_t band_num, int fd, int offset, 
                    int size, int inode_num, long int age);
void append_deletelog(band_t band_num, int fd, int offset, 
                      int size, int entrynumber);
struct store_entry* get_livelog(int band_num, int store_fd, int delete_fd,
                                int *length, int *livedatablocks);
int band_delentries(int band_num);
void get_livelog_wrapper(band_t, struct store_entry**, int *);
int recover_band_log_data();

#ifdef __cplusplus
}
#endif
#endif /* __BAND_LOG_H_ */
