/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file cleaning_thread.h
 *
 * @brief Defines function prototypes pertaining to band cleaner
 *
 * @author Jainam Shah <jashah@andrew.cmu.edu>
 */

#ifndef __BAND_CLEAN_H_
#define __BAND_CLEAN_H_
#ifdef __cplusplus
extern "C" {
#endif

#include "common.h"
/**
 * Thresholds above which we start considering the cleaning.
 * Measured in terms of %age of filesystem consumed
 * soft - can consider cleaning
 * hard - have to start cleaning
 */
#define FS_FULL_THRESHOLD_SOFT 67
#define FS_FULL_THRESHOLD_HARD 90

typedef struct {
    band_t band_to_clean;
    band_t band_clean_to;
} band_clean_t;

typedef enum {
    INVALID_POLICY = 0,
    GET_CLEAN_BAND_FIRST,
    GET_BAND_TO_CLEAN_FIRST,
    COST_BENEFIT,
    GREEDY_BY_MAX_SPACE_RECOVERED,
    GREEDY_BY_MIN_EFFORT,
    CLUSTERED_CLEANING_BY_PRESERVING_NAMESPACE
} band_cleaning_policy_e;

typedef enum {
    INVALID_STOP_POLICY = 0,
    FIXED_THRESHOLD,
    MAX_FREE_SIZE
} time_to_stop_policy_e;


/* ========================= */
/* -- Function Prototypes -- */
/* ========================= */

int cleaner_init(void);
void *band_cleaner();
int band_cleaning();
void migrate_data(int band_frag, int band_clean, struct store_entry *live, int length);
void wait_cleaner(void);
void signal_cleaner(void);
void init_semcleaner(void);
int band_cleanall(int);
void band_cleanall_wrapper(int);
void copy_data(int band_src, int band_dst, int offset_src, int len_src,
               int offset_dst);
int select_cleaning_band(band_clean_t* band, band_cleaning_policy_e policy);
int time_to_stop(int cleaned_count, int, time_to_stop_policy_e policy);

#ifdef __cplusplus
}
#endif
#endif /* __BAND_CLEAN_H_ */
