/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file common.h
 *
 * @brief Common data structures and macros
 *
 * @author Anand Suresh <anandsuresh@cmu.edu>
 */

#ifndef __COMMON_H_
#define __COMMON_H_

#include <fuse.h>
#include <limits.h>

#include "log.h"
#include "emutypes.h"

#define TRUE 1
#define FALSE 0


/* ======================= */
/* -- Macro Definitions -- */
/* ======================= */

/**
 * @brief The prefix for attribute replication files
 */
#define FILE_PREFIX                                             ".smr_"
#define FILE_XATTR_PREFIX                               ".smr_xattr_"
#define FILE_INODE_XATTR_PREFIX				".smr_inode_xattr_"
#define MEMALIGN_SIZE					512


/* ====================== */
/* -- Type Definitions -- */
/* ====================== */

typedef struct {
    char                         path_unshingled[PATH_MAX];
    char                         path_tmpfs[PATH_MAX];
    char                         path_edi[PATH_MAX];
    char			 path_edi_metadata[PATH_MAX];

    int                          fd_edi;
    int				 fd_edi_metadata;
    int                          fd_bitmap;

    uint32_t             edi_blksize;
    band_t                       edi_bandcount;
    rba_t                        edi_bandsize;
}shingledfs_data_t;



/* ====================== */
/* -- Global Variables -- */
/* ====================== */

/**
 * @brief Stores ShingledFS global data including paths, open file descriptors,
 * and EDI parameters.
 */
extern shingledfs_data_t shingledfs_data;
#define SFSDATA shingledfs_data

int max_inode_used;

#endif  /* __COMMON_H_ */
