/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file helper.h
 *
 * @brief Helper functions, data types and macros
 *
 * @author Anand Suresh <anandsur@andrew.cmu.edu>
 */

#ifndef __HELPER_H_
#define __HELPER_H_
#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include <limits.h>
#include <pthread.h>

#include "debug.h"
#include "common.h"
#include "inode.h"



/* ======================= */
/* -- Macro Definitions -- */
/* ======================= */

#define X_STATS                         "user.sfs.stats"
#define X_INODE                         "user.sfs.inode"
#define X_INODE_STRUCT			"user.sfs.inode.struct"



/* ====================== */
/* -- Type Definitions -- */
/* ====================== */



/* ========================= */
/* -- Function Prototypes -- */
/* ========================= */
void    read_edi_parameters(int fd, uint32_t *blk_size, band_t *band_count,
                            rba_t *band_size);
void    translate_path(const char *in_path, const char *prefix, char *out_path);
void    xattr_path(const char *in_path, char *out_path);
void    xattr_path_from_inode(const char *in_path, char *out_path, int inode_num);
int             xattr_get(const char *path, const char *name, void *value, size_t size);
void    xattr_set(const char *path, const char *name, void *value, size_t size,
                  int flags);
int             is_valid_dir(const char *path);
int             is_valid_file(const char *path);

inline void mark_for_cleaning(inode_t inode);
inline void inode_free(int inodeno);
inline void inode_xattr_free(int inodeno);
inline void inode_update(inode_t *inode, band_t band, rba_t rba, size_t size);
//inline void inode_unlink(inode_t inode);
inline int inode_unlink(int inodeno, inode_t inode);


#ifdef __cplusplus
}
#endif
#endif /* __HELPER_H_ */
