/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file inode.h
 *
 * @brief Defines types and function prototypes pertaining to Inodes
 *
 * @author Jainam Shah <jashah@andrew.cmu.edu>
 */

#ifndef __INODE_H_
#define __INODE_H_
#ifdef __cplusplus
 extern "C" {
#endif


/* ======================= */
/* -- Macro Definitions -- */
/* ======================= */

#define MAX_NUM_INODES  (1 << 20)
#define INODE_MAX_STRLEN 8     /* strlen(inodenum_) for MAX_NUM_INODES */



/* ====================== */
/* -- Type Definitions -- */
/* ====================== */

/**
 * @brief Inode structure use to store meta data of a file
 */

typedef struct {
    int avail;
    int inode_num;
    int entrynumber;
    long band_id;
    long band_offset;
    long size;
} inode_t;



/* ========================= */
/* -- Function Prototypes -- */
/* ========================= */

int get_inode (int inode_num, inode_t *node);
int put_inode (int inode_num, inode_t *node);
int get_offset(int inode_num);
int init_inodebitmap(void);
int init_inodelist(void);
int get_freeinodenum(void);
void put_freeinodenum(int inode_num);
void lock_inodelist(void);
void unlock_inodelist(void);


#ifdef __cplusplus
}
#endif
#endif /* __INODE_H_ */
