/**
 * Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file    stats.h
 * A module to collect/store the stats in ShingledFS.
 *
 * @author  Pavan Kumar Alampalli (pavankumar@cmu.edu)
 * @date    12-oct-2012
 */
//#include "band_meminfo.h"
#ifndef _STATS_H_
#define _STATS_H_
#ifdef __cplusplus
extern "C" {
#endif

#include "band_meminfo.h"

typedef enum {
    LAST_ACTIVITY_TIME,
    DISK_FREE_BLKS,
    DISK_MAX_BLKS,
    CLEANED_BLKS,
    CUM_CLEANED_BLKS,
    MOVED_BLKS,
    CUM_MOVED_BLKS,
    MAX_COUNTERS
} Counter;

typedef enum {
    BEFORE,
    AFTER
}Clean_time;

typedef enum {
    NEW_DATA,
    CLEAN_TO,
    DELETED,
    CLEANED
}State;

int *bands;

void stats_set(Counter c, unsigned long value);
void stats_incr(Counter c, unsigned long value);
void stats_decr(Counter c, unsigned long value);
unsigned long stats_get(Counter c);

int stats_update(void);
int stats_disk_utilization(void);
void stats_cleaner(Clean_time clean_time, int clean_flag, char * c);

#ifdef __cplusplus
}
#endif
#endif /* _STATS_H_ */
