#!/bin/bash
#
# Copyright 2013 Parallel Data Lab (Carnegie Mellon University)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# Ensure that the script has been passed the required no. of arguments
MAKE_TARGET="all"
if [ $# -gt 0 ]; then
    if [ $1 == "log" ]; then
        MAKE_TARGET="log"
    elif [ $1 == "debug" ]; then
        MAKE_TARGET="debug"
    elif [ $1 == "raw" ]; then
	MAKE_TARGET="raw"
    elif [ $1 == "raw-debug" ]; then
	MAKE_TARGET="raw-debug"
    fi
fi

# Define the variables
PATH_SRC="../../fuse"
PATH_EMULATOR="../../emulator"
PATH_BASE="/tmp/smr"
PATH_SMR=$PATH_BASE"/sfs"
PATH_TRN=$PATH_BASE"/trn"
PATH_USHNG=$PATH_BASE"/unshingled"
PATH_TRNBK=$PATH_BASE"/trnbk"
PATH_TMPFS=$PATH_BASE"/tmpfs"
if [ $MAKE_TARGET == "raw" ] || [ $MAKE_TARGET == "raw-debug" ]; then
    PATH_SHIMG_META=$PATH_BASE"/edi.meta"
    PATH_SHIMG="/dev/raw/raw1"
else
    PATH_SHIMG=$PATH_BASE"/edi.img"
fi

export STATSPATH=$PATH_BASE"/stats"

# Specify the working environment
echo "Base directory:                      " $PATH_BASE
echo "Transparent FUSE directory:          " $PATH_TRN
echo "Transparent FUSE backing directory : " $PATH_TRNBK
echo "SMR directory:                       " $PATH_SMR
echo "Unshingled directory:                " $PATH_USHNG
if [ $MAKE_TARGET == "raw" ] || [ $MAKE_TARGET == "raw-debug" ]; then
    echo "Shingled Metadata File:	   " $PATH_SHIMG_META
fi
echo "Shingled Image File:                 " $PATH_SHIMG
echo "tmpFS directory:                     " $PATH_TMPFS
echo "Target:                              " $MAKE_TARGET
echo


# Clean up the working environment
echo "Cleaning up..."
fusermount -u $PATH_SMR
fusermount -u $PATH_TRN
sudo umount $PATH_TMPFS
rm -rf $PATH_BASE/*
if [ $MAKE_TARGET == "log" ] || [ $MAKE_TARGET == "raw-debug" ] || [ $MAKE_TARGET == "debug" ]
then
    rm /tmp/smr*.log
fi
rm /tmp/smr*.csv
echo


# Make the executables
echo "Generating executables..."
make clean --directory=$PATH_SRC
make clean --directory=$PATH_EMULATOR

make -B $MAKE_TARGET --directory=$PATH_SRC
if [ $? -ne 0 ]; then
    echo "Error running test!"
    exit -1
fi
echo


# Setup the environment
echo "Setting up the environment..."

mkdir $PATH_BASE
mkdir $PATH_TRN
mkdir $PATH_TRNBK
mkdir $PATH_SMR
mkdir $PATH_USHNG
mkdir $PATH_TMPFS
mkdir $PATH_BASE/temp
mkdir $STATSPATH

sudo mount -t tmpfs -o size=2G tmpfs $PATH_TMPFS

rm -f *.img

if [ $MAKE_TARGET == "raw" ] || [ $MAKE_TARGET == "raw-debug" ]; then
../../emulator/edicreate -v 0 -s 4096 -c 33 -b 4096 -f $PATH_SHIMG -m $PATH_SHIMG_META
else
../../emulator/edicreate -v 0 -s 4096 -c 33 -b 4096 -f $PATH_SHIMG
fi

echo


# Launch the FUSE process
if [ $MAKE_TARGET == "raw-debug" ]; then
    gdb --args $PATH_SRC/shingledfs $PATH_USHNG $PATH_TMPFS $PATH_SHIMG $PATH_SHIMG_META $PATH_SMR -o big_writes -s
elif [ $MAKE_TARGET == "debug" ]; then
    gdb --args $PATH_SRC/shingledfs $PATH_USHNG $PATH_TMPFS $PATH_SHIMG $PATH_SMR -o big_writes -s
elif [ $MAKE_TARGET == "raw" ]; then
    $PATH_SRC/shingledfs $PATH_USHNG $PATH_TMPFS $PATH_SHIMG $PATH_SHIMG_META $PATH_SMR -o big_writes -s
else
    $PATH_SRC/shingledfs $PATH_USHNG $PATH_TMPFS $PATH_SHIMG $PATH_SMR -o big_writes -s
fi

$PATH_SRC/transparency $PATH_TRNBK $PATH_TRN -o big_writes -s

ps -e | grep shingledfs
ps -e | grep transparency
